@@ -1,15 +1,15 @@
-import logging
 import threading
 
 from .player import playerManager
 
+
 class ActionThread(threading.Thread):
     def __init__(self):
-        self.trigger        = threading.Event()
-        self.halt           = False
+        self.trigger = threading.Event()
+        self.halt = False
 
         threading.Thread.__init__(self)
-    
+
     def stop(self):
         self.halt = True
         self.join()
@@ -17,7 +17,7 @@ class ActionThread(threading.Thread):
     def run(self):
         force_next = False
         while not self.halt:
-            if (playerManager._player and playerManager._video) or force_next:
+            if playerManager.is_active() or force_next:
                 playerManager.update()
 
             force_next = False
@@ -25,5 +25,5 @@ class ActionThread(threading.Thread):
                 force_next = True
                 self.trigger.clear()
 
-actionThread = ActionThread()
 
+actionThread = ActionThread()
