@@ -3,7 +3,7 @@ from jellyfin_apiclient_python.connection_manager import CONNECTION_STATE
 from .conf import settings
 from . import conffile
 from getpass import getpass
-from .constants import CLIENT_VERSION, USER_APP_NAME, USER_AGENT, APP_NAME
+from .constants import CAPABILITIES, CLIENT_VERSION, USER_APP_NAME, USER_AGENT, APP_NAME
 from .i18n import _
 
 import sys
@@ -17,7 +17,10 @@ import re
 log = logging.getLogger("clients")
 path_regex = re.compile("^(https?://)?([^/:]+)(:[0-9]+)?(/.*)?$")
 
-def expo(max_value=None):
+from typing import Optional
+
+
+def expo(max_value: Optional[int] = None):
     n = 0
     while True:
         a = 2 ** n
@@ -27,11 +30,13 @@ def expo(max_value=None):
         else:
             yield max_value
 
+
 class ClientManager(object):
     def __init__(self):
         self.callback = lambda client, event_name, data: None
         self.credentials = []
         self.clients = {}
+        self.usernames = {}
         self.is_stopping = False
 
     def cli_connect(self):
@@ -55,23 +60,26 @@ class ClientManager(object):
             else:
                 log.warning(_("Adding server failed."))
 
-    def client_factory(self):
+    @staticmethod
+    def client_factory():
         client = JellyfinClient(allow_multiple_clients=True)
-        client.config.data['app.default'] = True
-        client.config.app(USER_APP_NAME, CLIENT_VERSION, settings.player_name, settings.client_uuid)
-        client.config.data['http.user_agent'] = USER_AGENT
-        client.config.data['auth.ssl'] = not settings.ignore_ssl_cert
+        client.config.data["app.default"] = True
+        client.config.app(
+            USER_APP_NAME, CLIENT_VERSION, settings.player_name, settings.client_uuid
+        )
+        client.config.data["http.user_agent"] = USER_AGENT
+        client.config.data["auth.ssl"] = not settings.ignore_ssl_cert
         return client
 
     def _connect_all(self):
         is_logged_in = False
         for server in self.credentials:
-            if self._connect_client(server):
+            if self.connect_client(server):
                 is_logged_in = True
         return is_logged_in
 
     def try_connect(self):
-        credentials_location = conffile.get(APP_NAME,'cred.json')            
+        credentials_location = conffile.get(APP_NAME, "cred.json")
         if os.path.exists(credentials_location):
             with open(credentials_location) as cf:
                 self.credentials = json.load(cf)
@@ -86,8 +94,11 @@ class ClientManager(object):
 
         is_logged_in = self._connect_all()
         if settings.connect_retry_mins and not is_logged_in:
-            log.warning("Connection failed. Will retry for {0} minutes."
-                        .format(settings.connect_retry_mins))
+            log.warning(
+                "Connection failed. Will retry for {0} minutes.".format(
+                    settings.connect_retry_mins
+                )
+            )
             for attempt in range(settings.connect_retry_mins * 2):
                 time.sleep(30)
                 is_logged_in = self._connect_all()
@@ -97,11 +108,16 @@ class ClientManager(object):
         return is_logged_in
 
     def save_credentials(self):
-        credentials_location = conffile.get(APP_NAME,'cred.json')
+        credentials_location = conffile.get(APP_NAME, "cred.json")
         with open(credentials_location, "w") as cf:
             json.dump(self.credentials, cf)
 
-    def login(self, server, username, password, force_unique=False):
+    def login(
+        self, server: str, username: str, password: str, force_unique: bool = False
+    ):
+        if server.endswith("/"):
+            server = server[:-1]
+        
         protocol, host, port, path = path_regex.match(server).groups()
 
         if not protocol:
@@ -110,7 +126,9 @@ class ClientManager(object):
 
         if protocol == "http://" and not port:
             log.warning("Adding port 8096 for insecure local http connection.")
-            log.warning("If you want to connect to standard http port 80, use :80 in the url.")
+            log.warning(
+                "If you want to connect to standard http port 80, use :80 in the url."
+            )
             port = ":8096"
 
         server = "".join(filter(bool, (protocol, host, port, path)))
@@ -128,23 +146,27 @@ class ClientManager(object):
             server["username"] = username
             if force_unique and server["Id"] in self.clients:
                 return True
-            self._connect_client(server)
+            self.connect_client(server)
             self.credentials.append(server)
             self.save_credentials()
             return True
         return False
 
-    def setup_client(self, client, server):
+    def setup_client(self, client: "JellyfinClient", server):
         def event(event_name, data):
-            if event_name == 'WebSocketDisconnect':
+            if event_name == "WebSocketDisconnect":
                 timeout_gen = expo(100)
                 if server["uuid"] in self.clients:
                     while not self.is_stopping:
                         timeout = next(timeout_gen)
-                        log.info("No connection to server. Next try in {0} second(s)".format(timeout))
+                        log.info(
+                            "No connection to server. Next try in {0} second(s)".format(
+                                timeout
+                            )
+                        )
                         self._disconnect_client(server=server)
                         time.sleep(timeout)
-                        if self._connect_client(server):
+                        if self.connect_client(server):
                             break
             else:
                 self.callback(client, event_name, data)
@@ -153,45 +175,37 @@ class ClientManager(object):
         client.callback_ws = event
         client.start(websocket=True)
 
-        client.jellyfin.post_capabilities({
-            'PlayableMediaTypes': "Video",
-            'SupportsMediaControl': True,
-            'SupportedCommands': (
-                "MoveUp,MoveDown,MoveLeft,MoveRight,Select,"
-                "Back,ToggleFullscreen,"
-                "GoHome,GoToSettings,TakeScreenshot,"
-                "VolumeUp,VolumeDown,ToggleMute,"
-                "SetAudioStreamIndex,SetSubtitleStreamIndex,"
-                "Mute,Unmute,SetVolume,DisplayContent,"
-                "Play,Playstate,PlayNext,PlayMediaSource"
-            ),
-        })
+        client.jellyfin.post_capabilities(CAPABILITIES)
 
-    def remove_client(self, uuid):
-        self.credentials = [server for server in self.credentials if server["uuid"] != uuid]
+    def remove_client(self, uuid: str):
+        self.credentials = [
+            server for server in self.credentials if server["uuid"] != uuid
+        ]
         self.save_credentials()
         self._disconnect_client(uuid=uuid)
 
-    def _connect_client(self, server):
+    def connect_client(self, server):
         if self.is_stopping:
             return False
 
         is_logged_in = False
         client = self.client_factory()
-        state = client.authenticate({"Servers":[server]})
-        server["connected"] = state['State'] == CONNECTION_STATE['SignedIn']
+        state = client.authenticate({"Servers": [server]}, discover=False)
+        server["connected"] = state["State"] == CONNECTION_STATE["SignedIn"]
         if server["connected"]:
             is_logged_in = True
             self.clients[server["uuid"]] = client
             self.setup_client(client, server)
+            if server.get("username"):
+                self.usernames[server["uuid"]] = server["username"]
 
         return is_logged_in
 
-    def _disconnect_client(self, uuid=None, server=None):
+    def _disconnect_client(self, uuid: Optional[str] = None, server=None):
         if uuid is None and server is not None:
             uuid = server["uuid"]
-            
-        if not uuid in self.clients:
+
+        if uuid not in self.clients:
             return
 
         if server is not None:
@@ -202,15 +216,33 @@ class ClientManager(object):
         client.stop()
 
     def remove_all_clients(self):
+        self.stop_all_clients()
+        self.credentials = []
+        self.save_credentials()
+
+    def stop_all_clients(self):
         for key, client in list(self.clients.items()):
             del self.clients[key]
             client.stop()
-        self.credentials = []
-        self.save_credentials()
 
     def stop(self):
         self.is_stopping = True
         for client in self.clients.values():
             client.stop()
 
+    def get_username_from_client(self, client):
+        # This is kind of convoluted. It may fail if a server
+        # was added before we started saving usernames.
+        for uuid, client2 in self.clients.items():
+            if client2 is client:
+                if uuid in self.usernames:
+                    return self.usernames[uuid]
+                for server in self.credentials:
+                    if server["uuid"] == uuid:
+                        return server.get("username", "Unknown")
+                break
+
+        return "Unknown"
+
+
 clientManager = ClientManager()
