@@ -28,7 +28,7 @@ mpv_log = logging.getLogger("mpv")
 discord_presence = False
 if settings.discord_presence:
     try:
-        from .rich_presence import send_presence, clear_presence
+        from .rich_presence import register_join_event, send_presence, clear_presence
 
         discord_presence = True
     except Exception:
@@ -181,6 +181,9 @@ class PlayerManager(object):
         self.menu = OSDMenu(self, self._player)
         self.syncplay = SyncPlayManager(self)
 
+        if discord_presence:
+            register_join_event(self.syncplay.discord_join_group)
+
         if hasattr(self._player, "osc"):
             self._player.osc = settings.enable_osc
         else:
@@ -622,8 +625,8 @@ class PlayerManager(object):
             if has_lock:
                 log.debug("PlayerManager::finished_callback starting next episode")
                 new_video = self._video.parent.get_next().video
+                self.send_timeline_stopped(True)
                 if self.syncplay.is_enabled():
-                    self.send_timeline_stopped(True)
                     self.syncplay.request_next(self._video.get_playlist_id())
                 else:
                     self.play(new_video)
@@ -661,8 +664,8 @@ class PlayerManager(object):
     def play_next(self):
         if self._video.parent.has_next:
             new_video = self._video.parent.get_next().video
+            self.send_timeline_stopped(True)
             if self.syncplay.is_enabled():
-                self.send_timeline_stopped(True)
                 self.syncplay.request_next(self._video.get_playlist_id())
             else:
                 self.play(new_video)
@@ -673,8 +676,8 @@ class PlayerManager(object):
     def skip_to(self, key: str):
         media = self._video.parent.get_from_key(key)
         if media:
+            self.send_timeline_stopped(True)
             if self.syncplay.is_enabled():
-                self.send_timeline_stopped(True)
                 self.syncplay.request_skip(media.video.get_playlist_id())
             else:
                 self.play(media.get_video(0))
@@ -685,8 +688,8 @@ class PlayerManager(object):
     def play_prev(self):
         if self._video.parent.has_prev:
             new_video = self._video.parent.get_prev().video
+            self.send_timeline_stopped(True)
             if self.syncplay.is_enabled():
-                self.send_timeline_stopped(True)
                 self.syncplay.request_prev(self._video.get_playlist_id())
             else:
                 self.play(new_video)
@@ -860,6 +863,7 @@ class PlayerManager(object):
                     player.playback_time,
                     player.duration,
                     not player.pause,
+                    self.syncplay.current_group,
                 )
             except Exception:
                 log.error("Could not send Discord Rich Presence.", exc_info=True)
